// Questo modulo  lo sviluppo di quello scritto da me (Oligoatria) e da
// Niccolo' (Shutdown) Ciardo per il progetto NETSAM

#include <windows.h>
#include <process.h>

#include "ThreadManager.h"


/* se il thread e' stato creato restituisce 1, altrimenti 0 */
/* uso: newThrd(id_thread, &funzione, (LPVOID)paramStruct) */
unsigned int cThreadManager::newThrd(unsigned int idThrd, void *pFunc, LPVOID lpParam) {
    /* usa _beginthreadex perche' CreateThread provoca Memory Leak */
    thread[idThrd].handle = (HANDLE)_beginthreadex(NULL, 0, (unsigned int (__stdcall *)(void *))pFunc, lpParam, 0,  &thread[idThrd].id);
    /* se il thread e' stato creato */
    if(thread[idThrd].handle) {
        /* cambio lo stato del thread */
        thread[idThrd].state = TDS_RUN;
        /* e ritorno un valore di successo */
        return 1;
    }
    /* altrimenti riporto errore */
    return 0;
}

/* questa funzione non restituisce nessun valore */
/* uso: closeThrd(id_thread) */
void cThreadManager::closeStopThisThrd(unsigned int idThrd) {
    /*
    Note: usa _endthreadex(0) per evitare Memory Leak.
          _endthreadex(0) chiude il thread corrente pero' dopo bisogna eliminare
          l'handle relativo al thread con CloseHandle(handle). a questo ci
          pensera' freeThrds() che verra' chiamato nel ciclo di routine. e'
          indispensabile chiamare questa funzione in quanto solo con questa si
          libera la memoria allocata quando si crea il thread.
    */
    thread[idThrd].state = TDS_DELETE; /* il thread dovra' essere fermato e eliminato */
    _endthreadex(0); /* chiudo il thread */
}

void cThreadManager::closeThrd(unsigned int idThrd) {   
    thread[idThrd].state = TDS_DELETE; /* il thread dovra' essere fermato e eliminato */	
}

/* questa funzione restituisce 1 in caso di successo, 0 in caso d'errore */
/* uso: freeThrd() */
unsigned int cThreadManager::freeThrds(void) {
    int res = 1, /* risultato delle operazioni */
        i; /* la variabile contatore per il for */
    /* incrementa di 1 finche' i<TD_MAX */
    for(i=0; i<TD_MAX; i++)
        /* se il thread deve essere eliminato */
        if(thread[i].state==TDS_DELETE)
            /* e se viene chiuso correttamente */
            if(CloseHandle(thread[i].handle))
                /* cambio lo stato del thread */
                thread[i].state = TDS_FREE;
            /* altrimenti */
            else
                /* ritorno errore */
                res = 0;
    /* ritorna il risultato dell'operazione */
    return res;
}

/* Restituisce 0 se fallisce, 1 se ha successo, 2 se non ha fatto niente */
/* uso: pauseThrd(id_thread) */
unsigned int cThreadManager::pauseThrd(int idThrd) {
    /* se il thread e' in esecuzione */
    if(thread[idThrd].state==TDS_RUN) {
        /* e se la sospensione del thread fallisce */
        if (SuspendThread(thread[idThrd].handle)==-1)
            /* restituisco errore */
            return 0;
        /* altrimenti, cambio lo stato del thread */
        thread[idThrd].state = TDS_SUSPEND;
        /* e percio' ritorno successo */
        return 1;
    }
    /* altrimenti */
    else
        /* se il thread e' sospeso */
        if (thread[idThrd].state==TDS_SUSPEND) {
            /* e se la ripresa del thread fallisce */
            if (ResumeThread(thread[idThrd].handle)==-1)
                /* riporto errore */
                return 0;
            /* altrimenti, cambio lo stato del thread */
            thread[idThrd].state = TDS_RUN;
            /* e riporto successo */
            return 1;
        }
    /* altrimenti se il thread e' da eliminare o non e' stato creato */
    return 2; /* ritorno successo, ma non faccio nulla */
}
